#include <stdio.h>
#include <stdarg.h>
#include <gtk/gtk.h>
#include <glade/glade.h>

#include "prefs.h"
#include "about.h"
#include "prefs_dlg.h"
#include "char_conv_dlg.h"
#include "dir_select.h"
#include "main_win.h"

/* resources */
#include "pixmaps/TagTool.xpm"


#define MAX_MSG_SIZE  256


/* private data */
static GtkWindow *w_main = NULL;
static GtkStatusbar* statusbar = NULL;

static int *width;
static int *height;

static gchar statusbar_msg[MAX_MSG_SIZE];
static guint statusbar_ctx = 0;


/* menu callbacks */
void cb_file_refresh(GtkWidget *widget, GdkEvent *event)
{
	ds_set_working_dir(NULL);
}

void cb_file_quit(GtkWidget *widget, GdkEvent *event)
{
	gtk_main_quit();
}

void cb_settings_charconv(GtkWidget *widget, GdkEvent *event)
{
	chconv_display(CHCONV_TAG);
}

void cb_settings_id3prefs(GtkWidget *widget, GdkEvent *event)
{
	prefs_display();
}

void cb_help_about(GtkWidget *widget, GdkEvent *event)
{
	about_display();
}


/* other UI callbacks */
void cb_main_win_size_changed(GtkWidget *widget, GtkAllocation *alloc, gpointer data)
{
	*width = alloc->width;
	*height = alloc->height;
}


/* public functions */
void mw_init(GladeXML *xml)
{
	GdkPixmap *xpm_icon;
	GdkBitmap *xpm_icon_m;


	/* 
	 * Tabs are visible by default to make editing in glade easier.
	 * They must be hidden before showing the window, otherwise they 
	 * would affect the window size.
	 */

	gtk_notebook_set_show_tabs(GTK_NOTEBOOK(glade_xml_get_widget(xml, "nb_file")), FALSE);
	gtk_notebook_set_show_tabs(GTK_NOTEBOOK(glade_xml_get_widget(xml, "nb_edit")), FALSE);
	gtk_notebook_set_show_tabs(GTK_NOTEBOOK(glade_xml_get_widget(xml, "nb_id3v1")), FALSE);
	gtk_notebook_set_show_tabs(GTK_NOTEBOOK(glade_xml_get_widget(xml, "nb_id3v2")), FALSE);
	gtk_notebook_set_show_tabs(GTK_NOTEBOOK(glade_xml_get_widget(xml, "nb_vorbis")), FALSE);


	/* 
	 * Hide the MP3 and Vorbis menus.  This has to be done here because 
	 * the specific MP3/Vorbis code may not be compiled.
	 */

	gtk_widget_hide(GTK_WIDGET(glade_xml_get_widget(xml, "m_id3")));
	gtk_widget_hide(GTK_WIDGET(glade_xml_get_widget(xml, "m_vorbis")));


	/*
	 * get the widgets from glade
	 */

	w_main = GTK_WINDOW(glade_xml_get_widget(xml, "w_main"));
	statusbar = GTK_STATUSBAR(glade_xml_get_widget(xml, "statusbar"));

	statusbar_ctx = gtk_statusbar_get_context_id(statusbar, "status_msg");


	/*
	 * get the preference values, or set them to defaults
	 */

	/* width */
	width = pref_get_ref("mw:width");
	if (!width) {
		gint temp = 0;
		width = pref_set("mw:width", PREF_INT, &temp);
	}

	/* height */
	height = pref_get_ref("mw:height");
	if (!height) {
		gint temp = 0;
		height = pref_set("mw:height", PREF_INT, &temp);
	}


	/*
	 * initialize the main window
	 */

	if (*width && *height)
		gtk_window_set_default_size(w_main, *width, *height);
	gtk_widget_show(GTK_WIDGET(w_main));


//	xpm_icon = gdk_pixmap_create_from_xpm(GTK_WIDGET(w_main)->window, &xpm_icon_m, 
//			NULL, "pixmaps/TagTool.xpm");
	xpm_icon = gdk_pixmap_create_from_xpm_d(GTK_WIDGET(w_main)->window, &xpm_icon_m, 
			NULL, TagTool_xpm);

	gdk_window_set_icon(GTK_WIDGET(w_main)->window, NULL, xpm_icon, xpm_icon_m);
}


void mw_status_printf(const gchar *format, ...)
{
	va_list ap;

	va_start(ap, format);
        vsnprintf(statusbar_msg, MAX_MSG_SIZE, format, ap);
	va_end(ap);
	statusbar_msg[MAX_MSG_SIZE-1] = 0;

	gtk_statusbar_pop(statusbar, statusbar_ctx);
	gtk_statusbar_push(statusbar, statusbar_ctx, statusbar_msg);
}


void mw_status_clear()
{
	gtk_statusbar_pop(statusbar, statusbar_ctx);
}

