#ifndef MODAL_DIALOG_H
#define MODAL_DIALOG_H


/* 
 * Creates a modal dialog box and waits for the user to close it. 
 * Returns an integer indicating which button was clicked.
 * NOTE: Only one modal_dialog can be visible at any given time.
 *
 * <transient>	If not NULL, the dialog is made transient for this window.
 * <title>	Dialog title.
 * <text>	Dialog message text.
 * <defbutton>	Default button (index starts at 1) or 0 for none.
 * <nbuttons>	Number of buttons.
 * <...>	Variable number of arguments of type char* with the button labels. 
 *		Number of arguments must match <nbuttons>.
 *
 * return	The index of the button that was clicked (starting at 1),
 *		or 0 if the dialog was closed by other means.
 */
int modal_dialog( GtkWindow *transientfor, 
		  char *title, 
		  char* text, 
		  int defbutton, 
		  int nbuttons, 
		  ... );


#endif
