#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <glade/glade.h>

#include "elist.h"
#include "gtk_util.h"
#include "mru.h"
#include "prefs.h"
#include "edit_tab.h"
#include "modal_dialog.h"
#include "cursor.h"
#include "genre.h"
#include "mpeg_file.h"
#include "mpeg_edit_field.h"

#include "mpeg_edit.h"

/* resources */
#include "pixmaps/graydot.xpm"
#include "pixmaps/greendot.xpm"



/* widgets we need */
static GtkWindow *w_main = NULL;
static GtkNotebook *nb_edit = NULL;
static GtkNotebook *nb_id3 = NULL;
static GtkNotebook *nb_id3v1 = NULL;
static GtkNotebook *nb_id3v2 = NULL;
static GtkMenuItem *m_id3 = NULL;
static GtkCheckMenuItem *m_id3v2_view_simple = NULL;
static GtkCheckMenuItem *m_id3v2_view_advanced = NULL;
static GtkMenuItem *m_id3_copyv1tov2 = NULL;
static GtkMenuItem *m_id3_copyv2tov1 = NULL;
static GtkButton *b_write_tag = NULL;
static GtkButton *b_remove_tag = NULL;

static GtkButton *b_id3v1_create_tag = NULL;
static GtkEntry *ent_id3v1_title = NULL;
static GtkEntry *ent_id3v1_artist = NULL;
static GtkEntry *ent_id3v1_album = NULL;
static GtkEntry *ent_id3v1_year = NULL;
static GtkEntry *ent_id3v1_comment = NULL;
static GtkSpinButton *spin_id3v1_track = NULL;
static GtkCombo *combo_id3v1_genre = NULL;
static GtkLabel *lab_id3v1_title = NULL;
static GtkLabel *lab_id3v1_artist = NULL;
static GtkLabel *lab_id3v1_album = NULL;
static GtkLabel *lab_id3v1_year = NULL;
static GtkLabel *lab_id3v1_comment = NULL;
static GtkLabel *lab_id3v1_track = NULL;
static GtkLabel *lab_id3v1_genre = NULL;

static GtkButton *b_id3v2_create_tag = NULL;
static GtkEntry *ent_id3v2_title = NULL;
static GtkEntry *ent_id3v2_artist = NULL;
static GtkEntry *ent_id3v2_album = NULL;
static GtkEntry *ent_id3v2_year = NULL;
static GtkText *text_id3v2_comment = NULL;
static GtkEntry *ent_id3v2_track = NULL;
static GtkCombo *combo_id3v2_genre = NULL;
static GtkLabel *lab_id3v2_title = NULL;
static GtkLabel *lab_id3v2_artist = NULL;
static GtkLabel *lab_id3v2_album = NULL;
static GtkLabel *lab_id3v2_year = NULL;
static GtkLabel *lab_id3v2_comment = NULL;
static GtkLabel *lab_id3v2_track = NULL;
static GtkLabel *lab_id3v2_genre = NULL;
static GtkLabel *lab_id3v2_advanced = NULL;

static GtkCList *clist_id3v2_frames = NULL;
static GtkButton *b_id3v2_add = NULL;
static GtkButton *b_id3v2_edit = NULL;
static GtkButton *b_id3v2_remove = NULL;

static GtkPixmap *pix_id3v1_tab = NULL;
static GtkPixmap *pix_id3v2_tab = NULL;
static GdkPixmap *xpm_graydot;
static GdkBitmap *xpm_graydot_m;
static GdkPixmap *xpm_greendot;
static GdkBitmap *xpm_greendot_m;


/* the various notebook tabs */
#define TAB_ID3_V1 0
#define TAB_ID3_V2 1
#define TAB_ID3V1_NOTAG 0
#define TAB_ID3V1_TAG 1
#define TAB_ID3V2_NOTAG 0
#define TAB_ID3V2_SIMPLE 1
#define TAB_ID3V2_ADVANCED 2

/* private data */
static int tab_edit_id3;
static gboolean changed_flag = TRUE;
static gboolean editable_flag = TRUE;
static gboolean ignore_changed_signals = FALSE;
static mpeg_file *file = NULL;

/* preferences */
static MRUList *genre_mru;
static int *current_tab;


/*** private functions ******************************************************/

static void set_changed_flag(gboolean value)
{
	if (changed_flag == value)
		return;

	changed_flag = value;
	gtk_widget_set_sensitive(GTK_WIDGET(b_write_tag), value);
}

static void set_editable_flag(gboolean value)
{
	if (editable_flag == value)
		return;

	editable_flag = value;
	gtk_widget_set_sensitive(GTK_WIDGET(b_remove_tag), value);

	/* v1 edit controls */
	gtk_widget_set_sensitive(GTK_WIDGET(b_id3v1_create_tag), value);
	gtk_widget_set_sensitive(GTK_WIDGET(ent_id3v1_title), value);
	gtk_widget_set_sensitive(GTK_WIDGET(lab_id3v1_title), value);
	gtk_widget_set_sensitive(GTK_WIDGET(ent_id3v1_artist), value);
	gtk_widget_set_sensitive(GTK_WIDGET(lab_id3v1_artist), value);
	gtk_widget_set_sensitive(GTK_WIDGET(ent_id3v1_album), value);
	gtk_widget_set_sensitive(GTK_WIDGET(lab_id3v1_album), value);
	gtk_widget_set_sensitive(GTK_WIDGET(ent_id3v1_year), value);
	gtk_widget_set_sensitive(GTK_WIDGET(lab_id3v1_year), value);
	gtk_widget_set_sensitive(GTK_WIDGET(ent_id3v1_comment), value);
	gtk_widget_set_sensitive(GTK_WIDGET(lab_id3v1_comment), value);
	gtk_widget_set_sensitive(GTK_WIDGET(combo_id3v1_genre), value);
	gtk_widget_set_sensitive(GTK_WIDGET(lab_id3v1_genre), value);
	gtk_widget_set_sensitive(GTK_WIDGET(spin_id3v1_track), value);
	gtk_widget_set_sensitive(GTK_WIDGET(lab_id3v1_track), value);
	gtk_widget_set_sensitive(GTK_WIDGET(m_id3_copyv1tov2), value);

	/* v2 simple edit controls */
	gtk_widget_set_sensitive(GTK_WIDGET(b_id3v2_create_tag), value);
	gtk_widget_set_sensitive(GTK_WIDGET(ent_id3v2_title), value);
	gtk_widget_set_sensitive(GTK_WIDGET(lab_id3v2_title), value);
	gtk_widget_set_sensitive(GTK_WIDGET(ent_id3v2_artist), value);
	gtk_widget_set_sensitive(GTK_WIDGET(lab_id3v2_artist), value);
	gtk_widget_set_sensitive(GTK_WIDGET(ent_id3v2_album), value);
	gtk_widget_set_sensitive(GTK_WIDGET(lab_id3v2_album), value);
	gtk_widget_set_sensitive(GTK_WIDGET(ent_id3v2_year), value);
	gtk_widget_set_sensitive(GTK_WIDGET(lab_id3v2_year), value);
	gtk_widget_set_sensitive(GTK_WIDGET(text_id3v2_comment), value);
	gtk_widget_set_sensitive(GTK_WIDGET(lab_id3v2_comment), value);
	gtk_widget_set_sensitive(GTK_WIDGET(combo_id3v2_genre), value);
	gtk_widget_set_sensitive(GTK_WIDGET(lab_id3v2_genre), value);
	gtk_widget_set_sensitive(GTK_WIDGET(ent_id3v2_track), value);
	gtk_widget_set_sensitive(GTK_WIDGET(lab_id3v2_track), value);
	gtk_widget_set_sensitive(GTK_WIDGET(m_id3_copyv2tov1), value);

	/* v2 advanced edit controls */
	gtk_widget_set_sensitive(GTK_WIDGET(clist_id3v2_frames), value);
	gtk_widget_set_sensitive(GTK_WIDGET(b_id3v2_add), value);
	gtk_widget_set_sensitive(GTK_WIDGET(b_id3v2_edit), value);
	gtk_widget_set_sensitive(GTK_WIDGET(b_id3v2_remove), value);
}

static void update_form_v1()
{
	const gchar *str;

	if (mpeg_file_has_tag_v(file, ID3TT_ID3V1)) {
		gtk_notebook_set_page(nb_id3v1, TAB_ID3V1_TAG);

		ignore_changed_signals = TRUE;
		mpeg_file_get_field_v(file, ID3TT_ID3V1, AF_TITLE, &str);
		gtk_entry_set_text(ent_id3v1_title, str);
		mpeg_file_get_field_v(file, ID3TT_ID3V1, AF_ARTIST, &str);
		gtk_entry_set_text(ent_id3v1_artist, str);
		mpeg_file_get_field_v(file, ID3TT_ID3V1, AF_ALBUM, &str);
		gtk_entry_set_text(ent_id3v1_album, str);
		mpeg_file_get_field_v(file, ID3TT_ID3V1, AF_YEAR, &str);
		gtk_entry_set_text(ent_id3v1_year, str);
		mpeg_file_get_field_v(file, ID3TT_ID3V1, AF_COMMENT, &str);
		gtk_entry_set_text(ent_id3v1_comment, str);
		mpeg_file_get_field_v(file, ID3TT_ID3V1, AF_TRACK, &str);
		gtk_spin_button_set_value(spin_id3v1_track, atoi(str));
		mpeg_file_get_field_v(file, ID3TT_ID3V1, AF_GENRE, &str);
		gtk_entry_set_text(GTK_ENTRY(combo_id3v1_genre->entry), str);
		ignore_changed_signals = FALSE;
	}
	else {
		gtk_notebook_set_page(nb_id3v1, TAB_ID3V1_NOTAG);
	}

	set_editable_flag(audio_file_is_editable((audio_file *)file));
}

static void update_form_v2()
{
	const gchar *str;

	if (mpeg_file_has_tag_v(file, ID3TT_ID3V2)) {
		int pos = 0;

		gtk_notebook_set_page(nb_id3v2, *current_tab);
		
		if (*current_tab == TAB_ID3V2_SIMPLE) {
			int extra_fields;
			ID3Frame **frames;
			int nframes;
			int i;

			ignore_changed_signals = TRUE;
			if (g_elist_length(genre_mru->list) > 0)
				gtk_combo_set_popdown_strings(combo_id3v2_genre, GLIST(genre_mru->list));
			mpeg_file_get_field_v(file, ID3TT_ID3V2, AF_TITLE, &str);
			gtk_entry_set_text(ent_id3v2_title, str);
			mpeg_file_get_field_v(file, ID3TT_ID3V2, AF_ARTIST, &str);
			gtk_entry_set_text(ent_id3v2_artist, str);
			mpeg_file_get_field_v(file, ID3TT_ID3V2, AF_ALBUM, &str);
			gtk_entry_set_text(ent_id3v2_album, str);
			mpeg_file_get_field_v(file, ID3TT_ID3V2, AF_YEAR, &str);
			gtk_entry_set_text(ent_id3v2_year, str);
			mpeg_file_get_field_v(file, ID3TT_ID3V2, AF_COMMENT, &str);
			gtk_editable_delete_text(GTK_EDITABLE(text_id3v2_comment), 0, -1);
			gtk_editable_insert_text(GTK_EDITABLE(text_id3v2_comment), str, strlen(str), &pos);
			mpeg_file_get_field_v(file, ID3TT_ID3V2, AF_TRACK, &str);
			gtk_entry_set_text(ent_id3v2_track, str);
			mpeg_file_get_field_v(file, ID3TT_ID3V2, AF_GENRE, &str);
			gtk_entry_set_text(GTK_ENTRY(combo_id3v2_genre->entry), str);
			ignore_changed_signals = FALSE;

			/* Show in the "Advanced" button label how many fields are not visible */
			extra_fields = 0;
			nframes = mpeg_file_get_frames(file, ID3TT_ID3V2, &frames);
			for (i = 0; i < nframes; i++) {
				gboolean is_simple;
				mpeg_file_get_frame_info(ID3Frame_GetID(frames[i]), NULL, NULL, NULL, &is_simple);
				if (!is_simple)
					extra_fields++;
			}

			if (extra_fields > 0) {
				char str[16];
				snprintf(str, sizeof(str), "Advanced (%i)", extra_fields);
				gtk_label_set_text(lab_id3v2_advanced, str);
			}
			else {
				gtk_label_set_text(lab_id3v2_advanced, "Advanced");
			}
		}
		else {
			ID3Frame **frames;
			int nframes;
			int i;

			nframes = mpeg_file_get_frames(file, ID3TT_ID3V2, &frames);
			
			/* fill in the clist */
			gtk_clist_freeze(clist_id3v2_frames);
			gtk_clist_clear(clist_id3v2_frames);

			for (i = 0; i < nframes; i++) {
				char buffer[64];
				const char *desc;
				const char *p;
				const char *row[2];
				int rownum;

				mpeg_file_get_frame_info(ID3Frame_GetID(frames[i]), NULL, &desc, NULL, NULL);
				p = strchr(desc, '/');
				snprintf(buffer, sizeof(buffer), "%.*s", (p ? p-desc : sizeof(buffer)), desc);
				
				row[0] = buffer;
				mpeg_file_get_frame_text(file, ID3TT_ID3V2, frames[i], &row[1]);
				if (row[1] == NULL)
					row[1] = "[non-text field]";

				rownum = gtk_clist_append(clist_id3v2_frames, (char**)row);
				gtk_clist_set_row_data(clist_id3v2_frames, rownum, frames[i]);
			}
			
			gtk_clist_set_sort_column(clist_id3v2_frames, 0);
			gtk_clist_sort(clist_id3v2_frames);
			gtk_clist_set_column_width(clist_id3v2_frames, 0, 
						   gtk_clist_optimal_column_width(clist_id3v2_frames, 0) + 4);
			gtk_clist_set_column_width(clist_id3v2_frames, 1, 
						   gtk_clist_optimal_column_width(clist_id3v2_frames, 1));

			gtk_clist_thaw(clist_id3v2_frames);

			free(frames);
		}
	}
	else {
		gtk_notebook_set_page(nb_id3v2, TAB_ID3V2_NOTAG);
	}

	set_editable_flag(audio_file_is_editable((audio_file *)file));
}

static void update_tag_from_form_v1(int tag)
{
	mpeg_file_set_field_v(file, tag, AF_TITLE, gtk_entry_get_text(ent_id3v1_title));
	mpeg_file_set_field_v(file, tag, AF_ARTIST, gtk_entry_get_text(ent_id3v1_artist));
	mpeg_file_set_field_v(file, tag, AF_ALBUM, gtk_entry_get_text(ent_id3v1_album));
	mpeg_file_set_field_v(file, tag, AF_YEAR, gtk_entry_get_text(ent_id3v1_year));
	mpeg_file_set_field_v(file, tag, AF_COMMENT, gtk_entry_get_text(ent_id3v1_comment));
	mpeg_file_set_field_v(file, tag, AF_TRACK, gtk_entry_get_text(GTK_ENTRY(spin_id3v1_track)));
	mpeg_file_set_field_v(file, tag, AF_GENRE, gtk_entry_get_text(GTK_ENTRY(combo_id3v1_genre->entry)));
}

static void update_tag_from_form_v2(int tag)
{
	if (*current_tab == TAB_ID3V2_SIMPLE) {
		mpeg_file_set_field_v(file, tag, AF_TITLE, gtk_entry_get_text(ent_id3v2_title));
		mpeg_file_set_field_v(file, tag, AF_ARTIST, gtk_entry_get_text(ent_id3v2_artist));
		mpeg_file_set_field_v(file, tag, AF_ALBUM, gtk_entry_get_text(ent_id3v2_album));
		mpeg_file_set_field_v(file, tag, AF_YEAR, gtk_entry_get_text(ent_id3v2_year));
		mpeg_file_set_field_v(file, tag, AF_COMMENT, gtk_editable_get_chars(GTK_EDITABLE(text_id3v2_comment), 0, -1));
		mpeg_file_set_field_v(file, tag, AF_TRACK, gtk_entry_get_text(ent_id3v2_track));
		mpeg_file_set_field_v(file, tag, AF_GENRE, gtk_entry_get_text(GTK_ENTRY(combo_id3v2_genre->entry)));
	}
	else {
		/* Nothing to do here, advanced mode alters tag directly */
	}
}

static void update_tag()
{
	if (mpeg_file_has_tag_v(file, ID3TT_ID3V1))
		update_tag_from_form_v1(ID3TT_ID3V1);

	if (mpeg_file_has_tag_v(file, ID3TT_ID3V2))
		update_tag_from_form_v2(ID3TT_ID3V2);
}

static void update_tabs(gboolean allow_change)
{
	if (mpeg_file_has_tag_v(file, ID3TT_ID3V1)) {
		gtk_pixmap_set(pix_id3v1_tab, xpm_greendot, xpm_greendot_m);
		gtk_widget_set_sensitive(GTK_WIDGET(m_id3_copyv1tov2), TRUE);
	}
	else {
		gtk_pixmap_set(pix_id3v1_tab, xpm_graydot, xpm_graydot_m);
		gtk_widget_set_sensitive(GTK_WIDGET(m_id3_copyv1tov2), FALSE);
	}

	if (mpeg_file_has_tag_v(file, ID3TT_ID3V2)) {
		gtk_pixmap_set(pix_id3v2_tab, xpm_greendot, xpm_greendot_m);
		gtk_widget_set_sensitive(GTK_WIDGET(m_id3_copyv2tov1), TRUE);
	}
	else {
		gtk_pixmap_set(pix_id3v2_tab, xpm_graydot, xpm_graydot_m);
		gtk_widget_set_sensitive(GTK_WIDGET(m_id3_copyv2tov1), FALSE);
	}

	gtk_notebook_set_page(nb_edit, tab_edit_id3);
	if (allow_change) {
		if (mpeg_file_has_tag_v(file, ID3TT_ID3V2))
			gtk_notebook_set_page(nb_id3, TAB_ID3_V2);
		else
			gtk_notebook_set_page(nb_id3, TAB_ID3_V1);
	}
}

static void write_to_file()
{
	int res;

	cursor_set_wait();

	if (mpeg_file_has_tag_v(file, ID3TT_ID3V2)) {
		char *genre = gtk_entry_get_text(GTK_ENTRY(combo_id3v2_genre->entry));
		if (strcmp(genre, "") != 0)
			mru_add(genre_mru, genre);
	}

	res = mpeg_file_write_changes(file);
	if (res != AF_OK) {
		char msg[128];
		snprintf(msg, 128, "Error saving file \"%s\":\n%s (%d)", 
			 file->name, strerror(errno), errno);
		modal_dialog(w_main, "Error Saving File", msg, 1, 1, "OK");
	}
	else {
		set_changed_flag(FALSE);
	}

	cursor_set_normal();
}


/* UI callbacks */
void cb_id3_write_changes(GtkButton *button, gpointer user_data)
{
	if (!file) {
		g_warning("file is not open!");
		return;
	}

	update_tag();
	write_to_file();
}

void cb_id3_remove_tag(GtkButton *button, gpointer user_data)
{
	if (!file) {
		g_warning("file is not open!");
		return;
	}

	if (gtk_notebook_get_current_page(nb_id3) == TAB_ID3_V1) {
		mpeg_file_remove_tag_v(file, ID3TT_ID3V1);
		update_form_v1();
	}
	else {
		mpeg_file_remove_tag_v(file, ID3TT_ID3V2);
		update_form_v2();
	}

	update_tabs(TRUE);
	set_changed_flag(TRUE);
}

void cb_id3v1_create_tag(GtkButton *button, gpointer user_data)
{
	if (!file) {
		g_warning("file is not open!");
		return;
	}

	mpeg_file_create_tag_v(file, ID3TT_ID3V1);
	set_changed_flag(TRUE);
	update_form_v1();
	update_tabs(FALSE);
}

void cb_id3v2_create_tag(GtkButton *button, gpointer user_data)
{
	if (!file) {
		g_warning("file is not open!");
		return;
	}

	mpeg_file_create_tag_v(file, ID3TT_ID3V2);
	set_changed_flag(TRUE);
	update_form_v2();
	update_tabs(FALSE);
}

/* v1 and v2 simple mode callbacks */
void cb_id3_tag_changed(GtkEditable *editable, gpointer user_data)
{
	if (!ignore_changed_signals)
		set_changed_flag(TRUE);
}

gboolean cb_id3_keypress(GtkWidget *widget, GdkEventKey *event, gpointer user_data)
{
	/* <return> saves the tag and advances to the next file */
	if (event->keyval == GDK_Return) {
		if (file && changed_flag) {
			update_tag();
			write_to_file();
		}
		et_advance_file();
		return FALSE;
	}

	return TRUE;
}

/* v2 advanced mode callbacks */
void cb_id3v2_frame_selected(GtkCList *clist, gint row, gint col, GdkEventButton *event, gpointer data)
{
	ID3Frame *frame = (ID3Frame*)gtk_clist_get_row_data(clist, row);
	int frame_id = ID3Frame_GetID(frame);
	gboolean editable;

	mpeg_file_get_frame_info(frame_id, NULL, NULL, &editable, NULL);
	
	gtk_widget_set_sensitive(GTK_WIDGET(b_id3v2_edit), editable);
	gtk_widget_set_sensitive(GTK_WIDGET(b_id3v2_remove), TRUE);
}

void cb_id3v2_frame_unselected(GtkCList *clist, gint row, gint col, GdkEventButton *event, gpointer data)
{
	gtk_widget_set_sensitive(GTK_WIDGET(b_id3v2_edit), FALSE);
	gtk_widget_set_sensitive(GTK_WIDGET(b_id3v2_remove), FALSE);
}

void cb_id3v2_add(GtkWidget *widget, GdkEvent *event)
{
	if ( mpeg_editfld_create_frame(file, ID3TT_ID3V2) ) {
		set_changed_flag(TRUE);
		update_form_v2();
	}
}

void cb_id3v2_edit(GtkWidget *widget, GdkEvent *event)
{
	int row = gtk_clist_get_first_selection(clist_id3v2_frames);
	if (row != -1) {
		ID3Frame *frame = (ID3Frame*)gtk_clist_get_row_data(clist_id3v2_frames, row);

		if ( mpeg_editfld_edit_frame(file, ID3TT_ID3V2, frame) ) {
			set_changed_flag(TRUE);
			update_form_v2();
		}
	}
}

void cb_id3v2_remove(GtkWidget *widget, GdkEvent *event)
{
	int row = gtk_clist_get_first_selection(clist_id3v2_frames);
	if (row != -1) {
		ID3Frame *frame = (ID3Frame*)gtk_clist_get_row_data(clist_id3v2_frames, row);
		mpeg_file_remove_frame(file, ID3TT_ID3V2, frame);

		set_changed_flag(TRUE);
		update_form_v2();
	}
}

void cb_id3v2_view_button(GtkWidget *widget, GdkEvent *event)
{
	if (*current_tab == TAB_ID3V2_SIMPLE)
		gtk_check_menu_item_set_active(m_id3v2_view_advanced, TRUE);
	else if (*current_tab == TAB_ID3V2_ADVANCED)
		gtk_check_menu_item_set_active(m_id3v2_view_simple, TRUE);
}

/* menu callbacks */
void cb_id3v2_view_simple(GtkWidget *widget, GdkEvent *event)
{
	if (*current_tab == TAB_ID3V2_ADVANCED) {
		*current_tab = TAB_ID3V2_SIMPLE;
		if (gtk_notebook_get_current_page(nb_id3v2) != TAB_ID3V2_NOTAG) {
			update_form_v2();
			gtk_notebook_set_page(nb_id3v2, TAB_ID3V2_SIMPLE);
		}
	}
}

void cb_id3v2_view_advanced(GtkWidget *widget, GdkEvent *event)
{
	if (*current_tab == TAB_ID3V2_SIMPLE) {
		update_tag_from_form_v2(ID3TT_ID3V2);

		*current_tab = TAB_ID3V2_ADVANCED;
		if (gtk_notebook_get_current_page(nb_id3v2) != TAB_ID3V2_NOTAG) {
			update_form_v2();
			gtk_notebook_set_page(nb_id3v2, TAB_ID3V2_ADVANCED);
		}
	}
}

void cb_id3_copyv1tov2(GtkWidget *widget, GdkEvent *event)
{
	if (!file) {
		g_warning("file is not open!");
		return;
	}

	if (!mpeg_file_has_tag_v(file, ID3TT_ID3V2))
		mpeg_file_create_tag_v(file, ID3TT_ID3V2);

	update_tag_from_form_v1(ID3TT_ID3V2);

	set_changed_flag(TRUE);
	update_form_v2();
	update_tabs(FALSE);
}

void cb_id3_copyv2tov1(GtkWidget *widget, GdkEvent *event)
{
	if (!file) {
		g_warning("file is not open!");
		return;
	}

	if (!mpeg_file_has_tag_v(file, ID3TT_ID3V1))
		mpeg_file_create_tag_v(file, ID3TT_ID3V1);

	update_tag_from_form_v2(ID3TT_ID3V1);

	set_changed_flag(TRUE);
	update_form_v1();
	update_tabs(FALSE);
}


/*** public functions *******************************************************/

void mpeg_edit_load(mpeg_file *f)
{
	gtk_widget_show(GTK_WIDGET(m_id3));

	file = f;
	set_changed_flag(FALSE);
	update_form_v1();
	update_form_v2();
	update_tabs(TRUE);
}


void mpeg_edit_unload()
{
	if (changed_flag) {
		int button;

		button = modal_dialog(w_main, "Save Changes", 
				      "ID3 Tag has been modified. Save changes?",
				      2, 2, "Save", "Discard");
		if (button == 1) {
			/* save button was pressed */
			update_tag();
			write_to_file();
		}
	}

	file = NULL;

	if (GTK_IS_WIDGET(m_id3))
		gtk_widget_hide(GTK_WIDGET(m_id3));
}


void mpeg_edit_init(GladeXML *xml)
{
	GEList *temp_list;
	GEList *genre_list;
	int default_tab = TAB_ID3V2_SIMPLE;

	w_main = GTK_WINDOW(glade_xml_get_widget(xml, "w_main"));
	nb_edit = GTK_NOTEBOOK(glade_xml_get_widget(xml, "nb_edit"));
	nb_id3 = GTK_NOTEBOOK(glade_xml_get_widget(xml, "nb_id3"));
	nb_id3v1 = GTK_NOTEBOOK(glade_xml_get_widget(xml, "nb_id3v1"));
	nb_id3v2 = GTK_NOTEBOOK(glade_xml_get_widget(xml, "nb_id3v2"));
	m_id3 = GTK_MENU_ITEM(glade_xml_get_widget(xml, "m_id3"));
	m_id3v2_view_simple = GTK_CHECK_MENU_ITEM(glade_xml_get_widget(xml, "m_id3v2_view_simple"));
	m_id3v2_view_advanced = GTK_CHECK_MENU_ITEM(glade_xml_get_widget(xml, "m_id3v2_view_advanced"));
	m_id3_copyv1tov2 = GTK_MENU_ITEM(glade_xml_get_widget(xml, "m_id3_copyv1tov2"));
	m_id3_copyv2tov1 = GTK_MENU_ITEM(glade_xml_get_widget(xml, "m_id3_copyv2tov1"));
	b_write_tag = GTK_BUTTON(glade_xml_get_widget(xml, "b_id3_write_tag"));
	b_remove_tag = GTK_BUTTON(glade_xml_get_widget(xml, "b_id3_remove_tag"));

	b_id3v1_create_tag = GTK_BUTTON(glade_xml_get_widget(xml, "b_id3v1_create_tag"));
	ent_id3v1_title = GTK_ENTRY(glade_xml_get_widget(xml, "ent_id3v1_title"));
	ent_id3v1_artist = GTK_ENTRY(glade_xml_get_widget(xml, "ent_id3v1_artist"));
	ent_id3v1_album = GTK_ENTRY(glade_xml_get_widget(xml, "ent_id3v1_album"));
	ent_id3v1_year = GTK_ENTRY(glade_xml_get_widget(xml, "ent_id3v1_year"));
	ent_id3v1_comment = GTK_ENTRY(glade_xml_get_widget(xml, "ent_id3v1_comment"));
	spin_id3v1_track = GTK_SPIN_BUTTON(glade_xml_get_widget(xml, "spin_id3v1_track"));
	combo_id3v1_genre = GTK_COMBO(glade_xml_get_widget(xml, "combo_id3v1_genre"));
	lab_id3v1_title = GTK_LABEL(glade_xml_get_widget(xml, "lab_id3v1_title"));
	lab_id3v1_artist = GTK_LABEL(glade_xml_get_widget(xml, "lab_id3v1_artist"));
	lab_id3v1_album = GTK_LABEL(glade_xml_get_widget(xml, "lab_id3v1_album"));
	lab_id3v1_year = GTK_LABEL(glade_xml_get_widget(xml, "lab_id3v1_year"));
	lab_id3v1_comment = GTK_LABEL(glade_xml_get_widget(xml, "lab_id3v1_comment"));
	lab_id3v1_track = GTK_LABEL(glade_xml_get_widget(xml, "lab_id3v1_track"));
	lab_id3v1_genre = GTK_LABEL(glade_xml_get_widget(xml, "lab_id3v1_genre"));

	b_id3v2_create_tag = GTK_BUTTON(glade_xml_get_widget(xml, "b_id3v2_create_tag"));
	ent_id3v2_title = GTK_ENTRY(glade_xml_get_widget(xml, "ent_id3v2_title"));
	ent_id3v2_artist = GTK_ENTRY(glade_xml_get_widget(xml, "ent_id3v2_artist"));
	ent_id3v2_album = GTK_ENTRY(glade_xml_get_widget(xml, "ent_id3v2_album"));
	ent_id3v2_year = GTK_ENTRY(glade_xml_get_widget(xml, "ent_id3v2_year"));
	text_id3v2_comment = GTK_TEXT(glade_xml_get_widget(xml, "text_id3v2_comment"));
	ent_id3v2_track = GTK_ENTRY(glade_xml_get_widget(xml, "ent_id3v2_track"));
	combo_id3v2_genre = GTK_COMBO(glade_xml_get_widget(xml, "combo_id3v2_genre"));
	lab_id3v2_title = GTK_LABEL(glade_xml_get_widget(xml, "lab_id3v2_title"));
	lab_id3v2_artist = GTK_LABEL(glade_xml_get_widget(xml, "lab_id3v2_artist"));
	lab_id3v2_album = GTK_LABEL(glade_xml_get_widget(xml, "lab_id3v2_album"));
	lab_id3v2_year = GTK_LABEL(glade_xml_get_widget(xml, "lab_id3v2_year"));
	lab_id3v2_comment = GTK_LABEL(glade_xml_get_widget(xml, "lab_id3v2_comment"));
	lab_id3v2_track = GTK_LABEL(glade_xml_get_widget(xml, "lab_id3v2_track"));
	lab_id3v2_genre = GTK_LABEL(glade_xml_get_widget(xml, "lab_id3v2_genre"));
	lab_id3v2_advanced = GTK_LABEL(glade_xml_get_widget(xml, "lab_id3v2_advanced"));

	clist_id3v2_frames = GTK_CLIST(glade_xml_get_widget(xml, "clist_id3v2_frames"));
	b_id3v2_add = GTK_BUTTON(glade_xml_get_widget(xml, "b_id3v2_add"));
	b_id3v2_edit = GTK_BUTTON(glade_xml_get_widget(xml, "b_id3v2_edit"));
	b_id3v2_remove = GTK_BUTTON(glade_xml_get_widget(xml, "b_id3v2_remove"));

	pix_id3v1_tab = GTK_PIXMAP(glade_xml_get_widget(xml, "pix_id3v1_tab"));
	pix_id3v2_tab = GTK_PIXMAP(glade_xml_get_widget(xml, "pix_id3v2_tab"));


	/* load the pixmaps */
	xpm_graydot = gdk_pixmap_create_from_xpm_d(GTK_WIDGET(w_main)->window, &xpm_graydot_m, 
						   NULL, graydot_xpm);
	xpm_greendot = gdk_pixmap_create_from_xpm_d(GTK_WIDGET(w_main)->window, &xpm_greendot_m, 
						    NULL, greendot_xpm);

	/* find out which tab has the ID3 interface */
	tab_edit_id3 = gtk_notebook_page_num(nb_edit, glade_xml_get_widget(xml, "cont_id3_edit"));

	/* ID3v1 genre list */
	temp_list = genre_create_list(TRUE);
	g_elist_prepend(temp_list, "");
	gtk_combo_set_popdown_strings(combo_id3v1_genre, GLIST(temp_list));
	g_elist_free(temp_list);

	/* ID3v2 genre list, from prefs */
	temp_list = g_elist_new();
	genre_list = pref_get_or_set("common_edit:genre_mru", PREF_STRING | PREF_LIST, temp_list);
	g_elist_free_data(temp_list);
	genre_mru = mru_new_from_list(50, genre_list);

	/* current_tab, from prefs */
	current_tab = pref_get_or_set("id3_edit:current_tab", PREF_INT, &default_tab);
	if (*current_tab != TAB_ID3V2_SIMPLE && *current_tab != TAB_ID3V2_ADVANCED)
		*current_tab = default_tab;

	if (*current_tab == TAB_ID3V2_SIMPLE)
		gtk_check_menu_item_set_active(m_id3v2_view_simple, TRUE);
	else if (*current_tab == TAB_ID3V2_ADVANCED)
		gtk_check_menu_item_set_active(m_id3v2_view_advanced, TRUE);
}

